/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2001  HASEGAWA Masafumi
 */

#include "ebi_pagemaker.h"
#include "ebi_textpage.h"
#include "ebi_line.h"
#include "ebi_textlink.h"
#include "ebi_imagelink.h"
#include "ebi_soundlink.h"
#include "ebi_word.h"
#include "ebi_char.h"
#include "ebi_gaiji.h"
#include "ebi_code.h"
#include "ebi_error.h"
#include "databox.h"

__EBI_BEGIN_NAMESPACE

PageMaker::PageMaker (TextPage * page)
{
  m_page = page;

  m_line = NULL;
  m_reference = NULL;
  m_word = NULL;
}

PageMaker::~PageMaker ()
{
}

void PageMaker::newLine ()
{
  m_line = NULL;
  m_reference = NULL;
  m_word = NULL;
}

void PageMaker::undoLine ()
{
  if (m_line != NULL)
    {
      m_page->deleteLine (m_line);
    }

  m_line = NULL;
  m_reference = NULL;
  m_word = NULL;
}

void PageMaker::createLine ()
{
  if (m_line == NULL)
    {
      m_line = new Line;
      m_page->addLine (m_line);

      m_reference = NULL;
      m_word = NULL;
    }
}

void PageMaker::addElement (Element * element)
{
  if (m_word != NULL)
    {
      m_word->addElement (element);
    }
  else if (m_reference != NULL)
    {
      m_reference->addElement (element);
    }
  else if (m_line != NULL)
    {
      m_line->addElement (element);
    }
}

void PageMaker::beginReference (Subbook * subbook)
{
  createLine ();
  m_reference = NULL;
  m_word = NULL;

  TextLink* textLink = new TextLink (subbook);
  addElement (textLink);
  m_reference = textLink;
}

void PageMaker::endReference (const EB_Position & pos)
{
  m_reference->setPos (pos);

  m_reference = NULL;
  m_word = NULL;
}

void PageMaker::beginWord ()
{
  createLine ();
  m_word = NULL;

  Word* word = new Word;
  addElement (word);
  m_word = word;
}

void PageMaker::endWord ()
{
  m_word = NULL;
}

void PageMaker::putChar (TC code, FontType fontType)
{
  createLine ();

  switch (fontType)
    {
    case FONT_HALF:
      switch (code)
	{
	case TK_SP0:
	case TK_KSP:
	  code = TK_KSP;
	  endWord ();
	  break;

	case TK_SP1:
	  code = TK_KSP;
	  break;

	default:
	  if (Code::isLatin1 (code))
	    {
	      if (m_word == NULL)
		{
		  beginWord ();
		}
	    }
	  else
	    {
	      endWord ();
	    }
	  break;
	}
      break;

    case FONT_FULL:
      endWord ();
      break;
    }

  addElement (new Char (code, fontType));
}

void PageMaker::putGaiji (Subbook* subbook, int code, FontType fontType)
{
  createLine ();

  switch (fontType)
    {
    case FONT_HALF:
      if (m_word == NULL)
	{
	  beginWord ();
	}
      break;

    case FONT_FULL:
      endWord ();
      break;
    }

  addElement (new Gaiji (subbook, code, fontType));
}

void PageMaker::putString (const String & str)
{
  for (String::const_iterator si = str.begin (); si != str.end (); ++si)
    {
      if (tc_isalnum (*si))
	{
	  putChar (*si, FONT_HALF);
	}
      else
	{
	  putChar (*si, FONT_FULL);
	}
    }
}

void PageMaker::putString (const String & str, FontType fontType)
{
  for (String::const_iterator si = str.begin (); si != str.end (); ++si)
    {
      putChar (*si, fontType);
    }
}

void PageMaker::beginGraphic (Subbook* subbook, ImageLink::ImageType type, int width, int height)
{
  createLine ();
  m_reference = NULL;
  m_word = NULL;

  ImageLink* imageLink = new ImageLink (subbook, type, width, height);
  addElement (imageLink);
  m_reference = imageLink;
}

void PageMaker::beginGraphic (Subbook* subbook, ImageLink::ImageType type, const EB_Position & pos)
{
  beginGraphic (subbook, type, 0, 0);

  m_reference->setPos (pos);
}

void PageMaker::endGraphic ()
{
  m_reference = NULL;
  m_word = NULL;
}

void PageMaker::endGraphic (const EB_Position & pos)
{
  m_reference->setPos (pos);

  endGraphic ();
}

void PageMaker::beginSound (Subbook* subbook, const EB_Position & pos, int size)
{
  createLine ();
  m_reference = NULL;
  m_word = NULL;

  SoundLink* soundLink = new SoundLink (subbook, size);
  addElement (soundLink);
  m_reference = soundLink;

  m_reference->setPos (pos);
}

void PageMaker::endSound ()
{
  m_reference = NULL;
  m_word = NULL;
}

__EBI_END_NAMESPACE
